/// Author:             Joe Audette
/// Created:            2006-01-20
/// Last Modified:      2009-09-19

using System;
using System.Globalization;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using mojoPortal.Business;
using mojoPortal.Business.WebHelpers;
using mojoPortal.Web.UI;
using mojoPortal.Web.Framework;

namespace mojoPortal.Web
{
    
    public partial class layout : MasterPage
    {
        private int leftModuleCount = 0;
        private int centerModuleCount = 0;
        private int rightModuleCount = 0;
        private SiteSettings siteSettings;
        private PageSettings currentPage;
        private SiteMapDataSource siteMapDataSource = null;
        private SiteMapNode rootNode = null;
        
        
        protected void Page_Load(object sender, EventArgs e)
        {
            siteSettings = CacheHelper.GetCurrentSiteSettings();
            currentPage = CacheHelper.GetCurrentPage();
            if (siteSettings == null) return;

            siteMapDataSource = (SiteMapDataSource)this.FindControl("SiteMapData");
            if(siteMapDataSource == null){ return;}

            siteMapDataSource.SiteMapProvider
                    = "mojosite" + siteSettings.SiteId.ToString(CultureInfo.InvariantCulture);

            try
            {
            rootNode = siteMapDataSource.Provider.RootNode;
            }
            catch(HttpException)
            {
                return;
            }

            SetupLayout();
        }

        private void SetupLayout()
        {
            if (!SiteUtils.IsNoMenuPage())
            {
                if ((SiteMenu1 != null)&& SiteMenu1.Visible)
                {
                    // printable view skin doesn't have a menu so it is null there
                    if (SiteMenu1.Parent.ID == "divLeft") leftModuleCount++;
                    if (SiteMenu1.Parent.ID == "divRight") rightModuleCount++;
                }

                Control c = this.FindControl("PageMenu1");
                if (
                    (c != null)
                    &&(c.Visible)
                    )
                {
                    PageMenuControl p = (PageMenuControl)c;
                    if (SiteUtils.TopPageHasChildren(rootNode, p.StartingNodeOffset))
                    {
                        if (c.Parent.ID == "divLeft") leftModuleCount++;
                        if (c.Parent.ID == "divRight") rightModuleCount++;
                    }
                    //else
                    //{
                    //    // if the menu is not visible hide the corner rounder too
                    //    if (p.Parent != null)
                    //    {
                    //        c = p.Parent.FindControl("ctop1");
                    //        if (c != null){ c.Visible = false;}

                    //        c = p.Parent.FindControl("cbottom1");
                    //        if (c != null) { c.Visible = false; }
                           
                    //    }
                    //}
                }

                c = this.FindControl("PageMenu2");
                if (
                    (c != null)
                    && (c.Visible)
                    )
                {
                    PageMenuControl p = (PageMenuControl)c;
                    if (SiteUtils.TopPageHasChildren(rootNode, p.StartingNodeOffset))
                    {
                        if (c.Parent.ID == "divLeft") leftModuleCount++;
                        if (c.Parent.ID == "divRight") rightModuleCount++;
                    }
                }

                c = this.FindControl("PageMenu3");
                if (
                    (c != null)
                    && (c.Visible)
                    )
                {
                    PageMenuControl p = (PageMenuControl)c;
                    if (SiteUtils.TopPageHasChildren(rootNode, p.StartingNodeOffset))
                    {
                        if (c.Parent.ID == "divLeft") leftModuleCount++;
                        if (c.Parent.ID == "divRight") rightModuleCount++;
                    }
                }

                c = this.FindControl("pnlMenu");
                if ((c != null) && (c.Parent.ID == "divLeft")) leftModuleCount++;

            }

            if ((currentPage != null)&&(IsContentSystemPage()))
            {
                foreach (Module module in currentPage.Modules)
                {
                    if (ModuleIsVisible(module))
                    {
                        if (StringHelper.IsCaseInsensitiveMatch(module.PaneName, "leftpane"))
                        {
                            leftModuleCount++;
                        }

                        if (StringHelper.IsCaseInsensitiveMatch(module.PaneName, "rightpane"))
                        {
                            rightModuleCount++;
                        }

                        if (StringHelper.IsCaseInsensitiveMatch(module.PaneName, "contentpane"))
                        {
                            centerModuleCount++;
                        }
                    }
                }
            }

            // this is to make room for ModuleWrapper or custom usercontrols if they exsits anywhere in left or right
            foreach(Control c in divRight.Controls)
            {
                if (c is mojoUserControl) { rightModuleCount++; }
            }

            foreach (Control c in divLeft.Controls)
            {
                if (c is mojoUserControl) { leftModuleCount++; }
            }

            if ((Request.RawUrl.Contains("MyPage.aspx"))
                        || (Request.RawUrl.Contains("ChooseContent.aspx"))
                        || (Request.RawUrl.Contains("SiteMail"))
                        )
            {
                this.divLeft.CssClass = "left-mypage";
                this.divCenter.CssClass = "center-mypage";
                this.divRight.CssClass = "right-mypage";
            }
            else
            {
                this.divLeft.Visible = (leftModuleCount > 0);
                this.divRight.Visible = (rightModuleCount > 0);

                if((divLeft.Visible)&&(!divRight.Visible))
                {
                    divLeft.CssClass = "leftside left2column cmszone";
                }

                if ((divRight.Visible) && (!divLeft.Visible))
                {
                    divRight.CssClass = "rightside right2column cmszone";
                }

                this.divCenter.CssClass =
                    this.divLeft.Visible
                        ? (this.divRight.Visible ? "center-rightandleftmargins cmszone" : "center-leftmargin cmszone")
                        : (this.divRight.Visible ? "center-rightmargin cmszone" : "center-nomargins cmszone");

                divLeft.CssClass += " cmszone";
                divLeft.CssClass += " cmszone";

                Control alt = this.FindControl("divAlt1");
                if ((alt != null) && (alt is Panel))
                {
                    ((Panel)alt).CssClass += " cmszone";
                }

                alt = this.FindControl("divAlt2");
                if ((alt != null) && (alt is Panel))
                {
                    ((Panel)alt).CssClass += " cmszone";
                }

                alt = this.FindControl("divAltContent2");
                if ((alt != null) && (alt is Panel))
                {
                    ((Panel)alt).CssClass += " cmszone";
                }

            }
        }

        private bool ModuleIsVisible(Module module)
        {
            if ((module.HideFromAuthenticated) && (Request.IsAuthenticated)) { return false; }
            if ((module.HideFromUnauthenticated) && (!Request.IsAuthenticated)) { return false; }

            return true;
        }

        private bool IsContentSystemPage()
        {
            if (Request.RawUrl.Contains("SiteMail")) return false;
            if (Request.RawUrl.Contains("DevAdmin")) return false;
            if (Request.CurrentExecutionFilePath.Contains("Default.aspx")) return true;
            if (Request.CurrentExecutionFilePath.Contains("default.aspx")) return true;
            if (Request.Url.ToString() == siteSettings.SiteRoot) return true;
            return false;
        }
    }
}
