﻿//  Author:                     Joe Audette
//  Created:                    2009-09-05
//	Last Modified:              2009-09-05
// 
// The use and distribution terms for this software are covered by the 
// Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
// which can be found in the file CPL.TXT at the root of this distribution.
// By using this software in any fashion, you are agreeing to be bound by 
// the terms of this license.
//
// You must not remove this notice, or any other, from this software.

using System;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Resources;

namespace mojoPortal.Web.UI
{
    /// <summary>
    /// enables the main javascript for discqus and optionally renders the html widget
    /// </summary>
    public class DisqusWidget : WebControl
    {
        private string siteShortName = string.Empty;

        public string SiteShortName
        {
            get { return siteShortName; }
            set { siteShortName = value; }
        }

        private bool renderCommentCountScript = false;

        public bool RenderCommentCountScript
        {
            get { return renderCommentCountScript; }
            set { renderCommentCountScript = value; }
        }

        private bool renderWidget = false;

        public bool RenderWidget
        {
            get { return renderWidget; }
            set { renderWidget = value; }
        }

        private bool renderPoweredBy = false;

        public bool RenderPoweredBy
        {
            get { return renderPoweredBy; }
            set { renderPoweredBy = value; }
        }

        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            if (siteShortName.Length == 0) { return; }

            if (!Visible) { return; }

            if((renderWidget)&&(Page.Request.Url.ToString().Contains("localhost")))
            {
                SetupLocalhost();
            }

            if (renderCommentCountScript) { SetupCommentCountScript(); }
        }

        protected override void Render(HtmlTextWriter writer)
        {
            //base.Render(writer);
            if (siteShortName.Length == 0) { return; }

            if (!Visible) { return; }

            //if (renderCommentCountScript) { SetupCommentCountScript(); }

            if (renderWidget) { RenderThreadWidget(writer); }
        }

        private void RenderThreadWidget(HtmlTextWriter writer)
        {
            writer.Write("<div id=\"disqus_thread\"></div>");
            writer.Write("<script type=\"text/javascript\" src=\"http://disqus.com/forums/" + siteShortName + "/embed.js\"></script>");
            writer.Write("<noscript><a href=\"http://" + siteShortName + ".disqus.com/?url=ref\">" + Resource.DisqusViewDiscussion + "</a></noscript>");

            if (renderPoweredBy)
            {
                writer.Write("<a href=\"http://disqus.com\" class=\"dsq-brlink\">blog comments powered by <span class=\"logo-disqus\">Disqus</span></a>");
            }
        }

        private void SetupLocalhost()
        {
            Page.ClientScript.RegisterClientScriptBlock(
                typeof(Page),
                "disquslocal",
                "<script type=\"text/javascript\"> var disqus_developer = 1; </script>");
        }

        private void SetupCommentCountScript()
        {

            StringBuilder script = new StringBuilder();

            script.Append("\n<script type=\"text/javascript\">");

            script.Append("\n");
            script.Append("//<![CDATA[");
            script.Append("\n");
            script.Append("(function() {");
            script.Append("var links = document.getElementsByTagName('a');");
            script.Append("var query = '?';");
            script.Append("for(var i = 0; i < links.length; i++) {");
            script.Append("if(links[i].href.indexOf('#disqus_thread') >= 0) {");
            script.Append("query += 'url' + i + '=' + encodeURIComponent(links[i].href) + '&';");
            script.Append("}");
            script.Append("}");

            script.Append("document.write('<script charset=\"utf-8\" type=\"text/javascript\" src=\"http://disqus.com/forums/" 
                + siteShortName + "/get_num_replies.js' + query + '\"></' + 'script>');");

            script.Append("})();");
            script.Append("\n");
            script.Append("//]]>");

            script.Append("\n</script>");

            Page.ClientScript.RegisterStartupScript(
                typeof(Page),
                "disqusmain",
                script.ToString());


        }

    }
}
