///	Author:					Joe Audette
///	Created:				2006-12-14
/// Last Modified:			2009-11-15

using System;
using System.Globalization;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using mojoPortal.Business;
using mojoPortal.Business.WebHelpers;
using mojoPortal.Web.Framework;
using Resources;

namespace mojoPortal.Web.UI
{
    public class ModuleTitleControl : WebControl, INamingContainer
    {
        #region Constructors

        public ModuleTitleControl()
		{
            //if (this.Site != null && this.Site.DesignMode) 
            //{
            //    this.Visible = false;
            //    return; 
            //}
            if (HttpContext.Current == null) { return; }

			EnsureChildControls();

            
		}

		#endregion

        #region Control Declarations

        protected Literal litModuleTitle;
        protected HyperLink lnkModuleSettings;
        protected HyperLink lnkModuleEdit;
        protected ImageButton ibPostDraftContentForApproval;
        protected ImageButton ibApproveContent;
        protected HyperLink lnkRejectContent;
        protected ImageButton ibCancelChanges;
        protected ClueTipHelpLink statusLink;

        #endregion

        private string literalExtraMarkup = string.Empty;
        private bool disabledModuleSettingsLink = false;
        private Module module = null;

        private string editUrl = string.Empty;
        private string editText = string.Empty;
        private bool useHTag = true;
        private bool canEdit = false;
        private bool forbidModuleSettings = false;
        private bool showEditLinkOverride = false;
        private bool enableWorkflow = false;
        private SiteModuleControl siteModule = null;
        private ContentWorkflowStatus workflowStatus = ContentWorkflowStatus.None;
        private string siteRoot = string.Empty;

        #region Public Properties

        public Module ModuleInstance
        {
            get { return module; }
            set { module = value; }
        }

        public string LiteralExtraMarkup
        {
            get { return literalExtraMarkup; }
            set { literalExtraMarkup = value; }
        }

        public string EditUrl
        {
            get { return editUrl; }
            set { editUrl = value; }

            
        }

        public string EditText
        {
            get { return editText; }
            set { editText = value; }
           
        }

        public bool UseH1Tag
        {
            get { return useHTag; }
            set { useHTag = value; }
            
        }

        public bool DisabledModuleSettingsLink
        {
            get { return disabledModuleSettingsLink; }
            set { disabledModuleSettingsLink = value; }
        }

        public bool CanEdit
        {
            get { return canEdit; }
            set { canEdit = value; }
            
        }

        public bool ShowEditLinkOverride
        {
            get { return showEditLinkOverride; }
            set { showEditLinkOverride = value; }
            
        }

        public ContentWorkflowStatus WorkflowStatus
        {
            get { return workflowStatus; }
            set { workflowStatus = value; }
        }

        #endregion

        private SiteModuleControl GetParentAsSiteModelControl(Control child)
        {
            if (HttpContext.Current == null) { return null; }

            if (child.Parent == null)
            {
                return null;
            }
            else if (child.Parent is SiteModuleControl)
            {
                return child.Parent as SiteModuleControl;
            }
            else
            {
                return GetParentAsSiteModelControl(child.Parent);
            }
        }

        protected override void Render(HtmlTextWriter writer)
        {
           
            if (HttpContext.Current == null) 
            {
                writer.Write("[" + this.ID + "]");
                return;
            }
            else
            {
                

                if (module != null)
                {
                    writer.Write("<a id='module" + module.ModuleId.ToString(CultureInfo.InvariantCulture) + "'></a>");
                }

                if (UseH1Tag)
                {
                    writer.WriteBeginTag("h2");
                    writer.WriteAttribute("class", "moduletitle");
                    writer.Write(HtmlTextWriter.TagRightChar);
                }

                litModuleTitle.RenderControl(writer);

                if (CanEdit)
                {
                    if (!forbidModuleSettings)
                    {
                        writer.Write(HtmlTextWriter.SpaceChar);
                        lnkModuleSettings.RenderControl(writer);
                    }

                    if (ibCancelChanges != null && ibCancelChanges.Visible)
                    {
                        writer.Write(HtmlTextWriter.SpaceChar);
                        ibCancelChanges.RenderControl(writer);
                    }

                    if (ibPostDraftContentForApproval != null && ibPostDraftContentForApproval.Visible)
                    {
                        writer.Write(HtmlTextWriter.SpaceChar);
                        ibPostDraftContentForApproval.RenderControl(writer);
                    }

                    if (lnkRejectContent != null && lnkRejectContent.Visible)
                    {
                        writer.Write(HtmlTextWriter.SpaceChar);
                        lnkRejectContent.RenderControl(writer);
                    }

                    if (ibApproveContent != null && ibApproveContent.Visible)
                    {
                        writer.Write(HtmlTextWriter.SpaceChar);
                        ibApproveContent.RenderControl(writer);
                    }

                    if (statusLink != null && statusLink.Visible)
                    {
                        writer.Write(HtmlTextWriter.SpaceChar);
                        statusLink.ToolTip = Resource.WorkflowStatus;
                        statusLink.RenderControl(writer);
                    }
                }

                if (
                    (lnkModuleEdit != null)
                    && (EditUrl != null)
                    && (EditText != null)
                    )
                {
                    writer.Write(HtmlTextWriter.SpaceChar);
                    lnkModuleEdit.RenderControl(writer);
                }

                if (literalExtraMarkup.Length > 0)
                {
                    writer.Write(literalExtraMarkup);
                }

                if (UseH1Tag)
                {
                    writer.WriteEndTag("h2");
                }


            }
            
            
        }

        void ibApproveContent_Click(object sender, ImageClickEventArgs e)
        {
            SiteModuleControl siteModule = GetParentAsSiteModelControl(this);
            if (siteModule == null) { return; }
            if (!(siteModule is IWorkflow)) { return; }

            IWorkflow workflow = siteModule as IWorkflow;
            workflow.Approve();

        }

        protected void ibPostDraftContentForApproval_Click(object sender, ImageClickEventArgs e)
        {
            SiteModuleControl siteModule = GetParentAsSiteModelControl(this);
            if (siteModule == null) { return; }
            if (!(siteModule is IWorkflow)) { return; }

            IWorkflow workflow = siteModule as IWorkflow;
            workflow.SubmitForApproval();
           
        }

        protected void ibCancelChanges_Click(object sender, ImageClickEventArgs e)
        {
            SiteModuleControl siteModule = GetParentAsSiteModelControl(this);
            if (siteModule == null) { return; }
            if (!(siteModule is IWorkflow)) { return; }

            IWorkflow workflow = siteModule as IWorkflow;
            workflow.CancelChanges();
           
        }

       

        protected override void OnPreRender(EventArgs e)
        {
           
            base.OnPreRender(e);
            if (HttpContext.Current == null) { return; }
            
            Initialize();
            

        }

        private void Initialize()
        {
            if (HttpContext.Current == null) { return; }
            
            if (this.Parent is SiteModuleControl)
            {
                siteModule = this.Parent as SiteModuleControl;
            }

            if (this.Parent is Panel)
            {
                if (this.Parent.Parent is SiteModuleControl)
                {
                    siteModule = this.Parent.Parent as SiteModuleControl;
                }
                else
                {
                    //this wil be true if using ModulePanel as a wrapper
                    if (this.Parent.Parent.Parent is SiteModuleControl)
                    {
                        siteModule = this.Parent.Parent.Parent as SiteModuleControl;
                    }
                
                }
            }

            if (siteModule != null)
            {
                module = siteModule.ModuleConfiguration;
                CanEdit = siteModule.IsEditable;
                enableWorkflow = siteModule.EnableWorkflow;
                forbidModuleSettings = siteModule.ForbidModuleSettings;
                  
            }

            if (module != null)
            {
                if (module.ShowTitle)
                {
                    litModuleTitle.Text = Page.Server.HtmlEncode(module.ModuleTitle);
                }
                else
                {
                    UseH1Tag = false;
                }

                if (CanEdit)
                {
                   
                    if (!disabledModuleSettingsLink)
                    {
                        lnkModuleSettings.Visible = true;
                        lnkModuleSettings.Text = Resource.SettingsLink;
                        lnkModuleSettings.ToolTip = Resource.ModuleEditSettings;

                        if ((WebConfigSettings.UseIconsForAdminLinks)&&(!WebConfigSettings.UseTextLinksForFeatureSettings))
                        {
                            lnkModuleSettings.ImageUrl = Page.ResolveUrl(WebConfigSettings.EditSettingsImage);
                        }
                        else
                        {
                            // if its a text link make it small like the edit link
                            lnkModuleSettings.CssClass = "ModuleEditLink";
                        }

                        siteRoot = SiteUtils.GetNavigationSiteRoot();

                        lnkModuleSettings.NavigateUrl = siteRoot
                            + "/Admin/ModuleSettings.aspx?mid=" + module.ModuleId.ToInvariantString()
                            + "&pageid=" + module.PageId.ToInvariantString();

                        if ((enableWorkflow) && (siteModule != null) && (siteModule is IWorkflow))
                        {
                            SetupWorkflowControls();
                            
                        }

                    }

                }

                if (
                    ((CanEdit) || (ShowEditLinkOverride))
                    && ((EditText != null) && (EditUrl != null)))
                {

                    lnkModuleEdit.Text = EditText;
                    if (this.ToolTip.Length > 0)
                    {
                        lnkModuleEdit.ToolTip = this.ToolTip;
                    }
                    else
                    {
                        lnkModuleEdit.ToolTip = EditText;
                    }
                    lnkModuleEdit.NavigateUrl = EditUrl
                        + "?mid=" + module.ModuleId.ToInvariantString()
                        + "&pageid=" + module.PageId.ToInvariantString();

                }

            }
        }

        private void SetupWorkflowControls()
        {
            if (HttpContext.Current == null) { return; }

            if (siteModule == null) { return; }
            if (module == null) { return; }

           
            CmsPage cmsPage = this.Page as CmsPage;
            if ((cmsPage != null) && (cmsPage.ViewMode == PageViewMode.WorkInProgress))
            {
                switch (workflowStatus)
                {
                    case ContentWorkflowStatus.Draft:

                        ibPostDraftContentForApproval.ImageUrl = Page.ResolveUrl(WebConfigSettings.RequestApprovalImage);
                        ibPostDraftContentForApproval.ToolTip = Resource.RequestApprovalToolTip;
                        ibPostDraftContentForApproval.Visible = true;
                        statusLink.HelpKey = "workflowstatus-draft-help";
                        break;

                    case ContentWorkflowStatus.AwaitingApproval:

                        //if (WebUser.IsAdminOrContentAdminOrContentPublisher)
                        if(
                            (cmsPage.CurrentPage != null)
                            &&(
                            (WebUser.IsInRoles(cmsPage.CurrentPage.EditRoles)) || (WebUser.IsInRoles(this.module.AuthorizedEditRoles))
                            )
                            )
                        {
                            //add in the reject and approve links:                                            
                            ibApproveContent.ImageUrl = Page.ResolveUrl(WebConfigSettings.ApproveContentImage);
                            ibApproveContent.Visible = true;
                            ibApproveContent.ToolTip = Resource.ApproveContentToolTip;

                            lnkRejectContent.NavigateUrl =
                                siteRoot
                                + "/Admin/RejectContent.aspx?mid=" + module.ModuleId.ToInvariantString()
                                + "&pageid=" + module.PageId.ToInvariantString();

                            lnkRejectContent.ImageUrl = Page.ResolveUrl(WebConfigSettings.RejectContentImage);
                            lnkRejectContent.ToolTip = Resource.RejectContentToolTip;
                            lnkRejectContent.Visible = true;
                        }

                        statusLink.HelpKey = "workflowstatus-awaitingapproval-help";

                        break;

                    case ContentWorkflowStatus.ApprovalRejected:
                        statusLink.HelpKey = "workflowstatus-rejected-help";
                        break;

                    
                }

                if (
                    (workflowStatus != ContentWorkflowStatus.Cancelled)
                    && (workflowStatus != ContentWorkflowStatus.Approved)
                    && (workflowStatus != ContentWorkflowStatus.None)
                    )
                {
                    //allow changes to be cancelled:                                            
                    ibCancelChanges.ImageUrl = Page.ResolveUrl(WebConfigSettings.CancelContentChangesImage);
                    ibCancelChanges.ToolTip = Resource.CancelChangesToolTip;
                    ibCancelChanges.Visible = true;
                }

            }
        }


        protected override void CreateChildControls()
        {
            if (HttpContext.Current == null) { return; }

            litModuleTitle = new Literal();
            //this.Controls.Add(litModuleTitle);
            lnkModuleSettings = new HyperLink();
            lnkModuleSettings.CssClass = "modulesettingslink";
            //this.Controls.Add(lnkModuleSettings);
            
            lnkModuleEdit = new HyperLink();
            //this.Controls.Add(lnkModuleEdit);
            lnkModuleEdit.CssClass = "ModuleEditLink";
            lnkModuleEdit.SkinID = "plain";


            ibPostDraftContentForApproval = new ImageButton();
            ibPostDraftContentForApproval.ID = "lbPostDraftContentForApproval";
            ibPostDraftContentForApproval.CssClass = "ModulePostDraftForApprovalLink";
            ibPostDraftContentForApproval.SkinID = "plain";
            ibPostDraftContentForApproval.Visible = false;
            ibPostDraftContentForApproval.Click += new ImageClickEventHandler(ibPostDraftContentForApproval_Click);
            this.Controls.Add(ibPostDraftContentForApproval);

            ibApproveContent = new ImageButton();
            ibApproveContent.ID = "ibApproveContent";
            ibApproveContent.CssClass = "ModuleApproveContentLink";
            ibApproveContent.SkinID = "plain";
            ibApproveContent.Visible = false;
            ibApproveContent.Click += new ImageClickEventHandler(ibApproveContent_Click);
            this.Controls.Add(ibApproveContent);

            lnkRejectContent = new HyperLink();
            lnkRejectContent.ID = "ibRejectContent";
            lnkRejectContent.CssClass = "ModuleRejectContentLink";
            lnkRejectContent.SkinID = "plain";
            lnkRejectContent.Visible = false;

            ibCancelChanges = new ImageButton();
            ibCancelChanges.ID = "ibCancelChanges";
            ibCancelChanges.CssClass = "ModuleCancelChangesLink";
            ibCancelChanges.SkinID = "plain";
            ibCancelChanges.Visible = false;
            UIHelper.AddConfirmationDialog(ibCancelChanges, Resource.CancelContentChangesButtonWarning);
            ibCancelChanges.Click += new ImageClickEventHandler(ibCancelChanges_Click);
            this.Controls.Add(ibCancelChanges);

            statusLink = new ClueTipHelpLink();
            
            this.Controls.Add(statusLink);


        }

        

    }
}
