/// Author:				    Joe Audette
/// Created:			    2004-08-26
///	Last Modified:		    2007-11-19
/// 
/// 2007-04-13   Alexander Yushchenko: made it WebControl instead of UserControl.
/// 2007-07-05   Alexander Yushchenko: added option to render as a simple heading.
/// 
/// The use and distribution terms for this software are covered by the 
/// Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
/// which can be found in the file CPL.TXT at the root of this distribution.
/// By using this software in any fashion, you are agreeing to be bound by 
/// the terms of this license.
///
/// You must not remove this notice, or any other, from this software.	

using System;
using System.Globalization;
using System.Web.UI;
using System.Web.UI.WebControls;
using mojoPortal.Business;
using mojoPortal.Business.WebHelpers;

namespace mojoPortal.Web.UI
{
    
    public class SiteTitle : WebControl
    {
        private string overrideUrl = string.Empty;
        public string OverrideUrl
        {
            get { return overrideUrl; }
            set { overrideUrl = value; }
        }

        private string overrideTitle = string.Empty;
        public string OverrideTitle
        {
            get { return overrideTitle; }
            set { overrideTitle = value; }
        }

        private bool useLink = true;
        public bool UseLink
        {
            get { return useLink; }
            set { useLink = value; }
        }


        protected override void Render(HtmlTextWriter writer)
        {
            if (this.Site != null && this.Site.DesignMode)
            {
                // TODO: show a bmp or some other design time thing?
                writer.Write("[" + this.ID + "]");
            }
            else
            {
                SiteSettings siteSettings = CacheHelper.GetCurrentSiteSettings();
                if (siteSettings == null) return;

                string titleText = (overrideTitle.Length > 0) ? overrideTitle : siteSettings.SiteName;
                string titleMarkup;

                if (useLink)
                {
                    string urlToUse;
                    if (overrideTitle.Length > 0 && overrideUrl.Length > 0)
                    {
                        urlToUse = (overrideUrl.StartsWith("~/") && siteSettings.SiteFolderName.Length > 0)
                                       ? SiteUtils.GetNavigationSiteRoot() + overrideUrl.Replace("~/", "/")
                                       : overrideUrl;
                    }
                    else
                    {
                        urlToUse = SiteUtils.GetNavigationSiteRoot() + "/Default.aspx";
                    }
                    titleMarkup = String.Format(CultureInfo.InvariantCulture, "<a class='siteheading' href='{0}'>{1}</a>",
                                                Page.ResolveUrl(urlToUse), titleText);
                }
                else
                {
                    titleMarkup = titleText;
                }

                writer.Write("<h1 class='siteheading'>{0}</h1>", titleMarkup);
            }
        }
    }
}
