﻿///	Author:				Joe Audette
///	Created:			2008-08-11
///	Last Modified:		2008-08-17
/// 
/// The use and distribution terms for this software are covered by the 
/// Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
/// which can be found in the file CPL.TXT at the root of this distribution.
/// By using this software in any fashion, you are agreeing to be bound by 
/// the terms of this license.
///
/// You must not remove this notice, or any other, from this software.

using System;
using System.Configuration;
using mojoPortal.Business;
using mojoPortal.Business.WebHelpers;
using mojoPortal.Web.Controls;
using Resources;


namespace mojoPortal.Web.UI
{
    public class mojoGoogleAnalyticsScript : GoogleAnalyticsScript
    {
        public const string mojoPageTrackerName = "mojoPageTracker";
        private SiteSettings siteSettings = null;
        private bool didInit = false;

        protected override void OnInit(EventArgs e)
        {
            base.OnInit(e);
            if (this.Site != null && this.Site.DesignMode) { return; }
            DoInit();
        }

        protected override void OnPreRender(EventArgs e)
        {
            if (!didInit) { DoInit(); }
            if (this.Site != null && this.Site.DesignMode) { return; }
            base.OnPreRender(e);

            

        }

        private void DoInit()
        {
            this.EnableViewState = false;

            didInit = true;
            // we need a known tracker name so we can call it from various places
            TrackerName = mojoPageTrackerName;
            MemberLabel = Resource.AnalyticsMember;
            CustomerLabel = Resource.AnalyticsCustomer;

            // lets always label Admins as admins, regardless whether they are also customers
            if (WebUser.IsAdmin)
            {
                MemberLabel = Resource.AnalyticsSiteAdminUser;
                CustomerLabel = Resource.AnalyticsSiteAdminUser;
            }

            TrackPageLoadTime = WebConfigSettings.TrackPageLoadTimeInGoogleAnalytics;
            LogToLocalServer = WebConfigSettings.LogGoogleAnalyticsDataToLocalWebLog;

            // do some setup before the base control runs
            // let Web.config setting trump site settings. this meets my needs where I want to track the demo site but am letting people login as admin
            // this way if the remove or change it in site settings it still uses my profile id
            if (ConfigurationSettings.AppSettings["GoogleAnalyticsProfileId"] != null)
            {
                GoogleAnalyticsProfileId = ConfigurationSettings.AppSettings["GoogleAnalyticsProfileId"].ToString();
                return;

            }

            siteSettings = CacheHelper.GetCurrentSiteSettings();
            if ((siteSettings != null) && (siteSettings.GoogleAnalyticsAccountCode.Length > 0))
            {
                GoogleAnalyticsProfileId = siteSettings.GoogleAnalyticsAccountCode;

            }
        }
    }
}
