﻿//	Created:			    2009-08-07
//	Last Modified:		    2009-08-07
// 
// The use and distribution terms for this software are covered by the 
// Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
// which can be found in the file CPL.TXT at the root of this distribution.
// By using this software in any fashion, you are agreeing to be bound by 
// the terms of this license.
//
// You must not remove this notice, or any other, from this software.	

using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Web.UI.WebControls;

namespace mojoPortal.Web.UI
{
    /// <summary>
    /// The purpose of this control is to provide a bridge so that we can move forward
    /// with Html 5 without breaking things for thos with existing skins targeting Xhtml
    /// who may not want to change right away.
    /// 
    /// By default a Panel renders as a div which allows things to stay compatible with Xhtml.
    /// Configuration options will allow this Panel to render as html 5 elements.
    /// 
    /// To use Html 5, the Web.config/user.config setting UseHtml5 should be set to true.
    /// 
    /// To continue using Xhtml, you can set it to false.
    /// 
    /// </summary>
    public class mojoPanel : Panel
    {
        private string tag = "div";
        private string overrideCss = string.Empty;

        /// <summary>
        /// Specify a tag to render as, expected values are the new html 5 elements header footer nav article section
        /// </summary>
        public string Tag
        {
            get { return tag; }
            set { tag = value; }
        }

        /// <summary>
        /// If you need to use different css for the overridden tag than for the non overriden div specify it here.
        /// </summary>
        public string OverrideCss
        {
            get { return overrideCss; }
            set { overrideCss = value; }
        }

        protected override void Render(System.Web.UI.HtmlTextWriter writer)
        {
            if ((!WebConfigSettings.UseHtml5)||(tag == "div") || (tag.Length == 0))
            {
                base.Render(writer);
                return;
            }

            if (overrideCss.Length > 0) { CssClass = overrideCss; }

            writer.Write("<" + tag + " class='" + CssClass + "'>\n");

            base.RenderContents(writer);

            writer.Write("\n</" + tag + "'>");


        }


    }
}
